PATTERN = r"BADGIE\s+TIME"

PATTERN_START = r"<!--\s+" + PATTERN + r"\s+-->"

PATTERN_END = r"<!--\s+END\s+" + PATTERN + r"\s+-->"

PATTERN_GIT_SSH = r"^(?P<user>git)@(?P<host>.*?):(?P<path>.*?)\.git$"



from ..models import Remote
from ._base import Badge, register_badge


@register_badge
class BrettOpsBadge(Badge):
    """
    Show that the repository is a BrettOps repository.
    """

    name = "brettops"

    remotes = (
        Remote(prefix="https://gitlab.com/brettops/containers/", name="container"),
        Remote(prefix="https://gitlab.com/brettops/packages/", name="package"),
        Remote(prefix="https://gitlab.com/brettops/pipelines/", name="pipeline"),
        Remote(prefix="https://gitlab.com/brettops/tools/", name="tool"),
        Remote(prefix="https://gitlab.com/brettops/ansible/roles/", name="role"),
    )

    def get_link_title(self):
        return f"brettops {self.remote.name}"

    def get_badge_image_url(self):
        return f"https://img.shields.io/badge/brettops-{self.remote.name}-209cdf?labelColor=162d50"

    def get_link_url(self):
        return "https://brettops.io"



import json
import os
import urllib.error
import urllib.request
from pathlib import Path

import gitlab

from ..models import Project

GITLAB_URL = "https://gitlab.com"
API_V4_URL = f"{GITLAB_URL}/api/v4"

gl = gitlab.Gitlab(private_token=os.environ.get("GITLAB_PRIVATE_TOKEN", None))

HEADERS: dict[str, str] = {}
private_token = os.environ.get("GITLAB_PRIVATE_TOKEN")
if private_token:
    HEADERS.update({"PRIVATE-TOKEN": private_token})


def get_project(remote_path):
    glproject = gl.projects.get(remote_path)

    project = Project(
        local_path=Path.cwd(),
        url=glproject.web_url,
        ref=glproject.default_branch,
        name=glproject.name,
        namespace=glproject.namespace,
        full_name=glproject.name_with_namespace,
        path=glproject.path,
        full_path=glproject.path_with_namespace,
    )

    return glproject, project


def get_latest_pipeline(glproject):
    try:
        return glproject.pipelines.get("latest", ref=glproject.default_branch)
    except gitlab.exceptions.GitlabGetError:
        return None


def get_latest_release(project_id):
    try:
        url = f"{API_V4_URL}/projects/{project_id}/releases/permalink/latest"
        request = urllib.request.Request(url=url, headers=HEADERS)
        response = urllib.request.urlopen(request)
        content = response.read()
        return json.loads(content)
    except urllib.error.HTTPError:
        return None



from dataclasses import dataclass
from pathlib import Path


@dataclass(frozen=True)
class Project:
    local_path: Path
    url: str
    ref: str
    name: str
    namespace: str
    path: str
    full_name: str
    full_path: str


@dataclass(frozen=True)
class Remote:
    name: str
    prefix: str



from ._base import Badge, register_badge


@register_badge
class PreCommitBadge(Badge):
    """
    Show that the repository is using pre-commit.
    """

    name = "pre-commit"
    link_title = "pre-commit"

    # Path.glob matching
    files = (".pre-commit-config.yaml",)

    def get_badge_image_url(self):
        return f"https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white"

    def get_link_url(self):
        return f"https://github.com/pre-commit/pre-commit"



from ._base import Badge, register_badge


@register_badge
class GitLabLatestReleaseBadge(Badge):
    """
    Show the latest GitLab release by date.
    """

    name = "gitlab-latest-release"
    link_title = "latest release"

    def get_badge_image_url(self):
        return f"https://img.shields.io/gitlab/v/release/{self.project.full_path}"

    def get_link_url(self):
        return f"{self.project.url}/-/releases"


@register_badge
class GitLabCoverageReportBadge(Badge):
    """
    Show the most recent coverage score on the default branch.
    """

    name = "gitlab-coverage-report"
    link_title = "coverage report"

    def get_badge_image_url(self):
        return f"{self.project.url}/badges/{self.project.ref}/coverage.svg"

    def get_link_url(self):
        return f"{self.project.url}/-/commits/{self.project.ref}"


@register_badge
class GitLabPipelineStatusBadge(Badge):
    """
    Show the most recent pipeline status on the default branch.
    """

    name = "gitlab-pipeline-status"
    link_title = "pipeline status"

    def get_badge_image_url(self):
        return f"{self.project.url}/badges/{self.project.ref}/pipeline.svg"

    def get_link_url(self):
        return f"{self.project.url}/-/commits/{self.project.ref}"



import sys
from dataclasses import dataclass
from typing import Optional

from termcolor import colored

from ..models import Project, Remote


@dataclass(frozen=True)
class Badge:
    project: Project
    remote: Optional[Remote] = None

    def __post_init__(self):
        print(
            colored(
                "- adding a {name} badge".format(
                    name=colored(self.__class__.name, "blue", attrs=["bold"])
                )
            ),
            file=sys.stderr,
        )

    def get_badge_image_url(self):
        raise NotImplementedError

    def get_link_title(self):
        try:
            return self.link_title
        except AttributeError:
            return ""

    def get_link_url(self):
        return self.project.url

    def get_markdown(self):
        return f"[![{self.get_link_title()}]({self.get_badge_image_url()})]({self.get_link_url()})"


_BADGES = {}


def register_badge(klass):
    assert klass.name not in _BADGES
    _BADGES[klass.name] = klass
    return klass


def get_badge(name):
    assert _BADGES[name]



import argparse
import logging
import re
import subprocess
import sys

from termcolor import colored

from ._version import __version__
from .badges._base import _BADGES
from .badges.brettops import BrettOpsBadge
from .badges.gitlab import (
    GitLabCoverageReportBadge,
    GitLabLatestReleaseBadge,
    GitLabPipelineStatusBadge,
)
from .badges.precommit import PreCommitBadge
from .constants import PATTERN_GIT_SSH
from .parser import parse_text
from .providers import gitlab as gitlab_provider
from .sources import get_badge_from_files, get_badge_from_remotes

RE_GIT_SSH = re.compile(PATTERN_GIT_SSH)

logging.basicConfig(level=logging.WARNING)


def get_badge_text(badges, format="markdown"):
    return "\n".join(getattr(badge, f"get_{format}")() for badge in badges)


class ListAction(argparse.Action):
    def __init__(
        self,
        option_strings,
        dest=argparse.SUPPRESS,
        default=argparse.SUPPRESS,
        help="list supported badges and exit",
    ):
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help,
        )

    def __call__(self, parser, _namespace, _values, _option_string=None):
        for name, badge in sorted(_BADGES.items(), key=lambda x: x[0]):
            # print(name, badge)
            print(
                "{name}: {description}".format(
                    name=colored(name, "cyan", attrs=["bold"]),
                    description=badge.__doc__.strip(),
                )
            )
        parser.exit()


def find_badges(text):
    badges = []
    process = subprocess.run(["git", "remote", "-v"], text=True, capture_output=True)
    url = process.stdout.splitlines()[0].split("\t")[1].split(" ")[0]
    match = RE_GIT_SSH.match(url)
    if match:
        if match.group("host") == "gitlab.com":
            glproject, project = gitlab_provider.get_project(match.group("path"))

            # add brettops badge
            new_badge = get_badge_from_remotes(
                badge_class=BrettOpsBadge, project=project
            )
            if new_badge is not None:
                badges.append(new_badge)

            # get gitlab latest release badge
            release = gitlab_provider.get_latest_release(glproject.id)
            if release:
                badges.append(
                    GitLabLatestReleaseBadge(
                        project=project,
                    )
                )

            # add gitlab latest pipeline badge
            glpipeline = gitlab_provider.get_latest_pipeline(glproject)
            if glpipeline is not None:
                badges.append(GitLabPipelineStatusBadge(project=project))

                # add gitlab coverage badge
                if glpipeline.coverage is not None:
                    badges.append(
                        GitLabCoverageReportBadge(
                            project=project,
                        )
                    )

            # add pre-commit badge
            new_badge = get_badge_from_files(
                badge_class=PreCommitBadge, project=project
            )
            if new_badge is not None:
                badges.append(new_badge)
    return badges


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--list", action=ListAction, help="list available badges")
    parser.add_argument(
        "-w", "--write", action="store_true", help="write changes to the file"
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument(
        "input",
    )

    args = parser.parse_args()

    text = open(args.input, "r").read()

    print(
        colored("Hi, I'm", "cyan", attrs=["bold"]),
        colored("Badgie!", "white", attrs=["bold"]),
        colored("Let's add some badges! 🐦\n", "cyan", attrs=["bold"]),
        file=sys.stderr,
    )

    badges = find_badges(text=text)
    badge_text = get_badge_text(badges=badges)
    output = parse_text(text, badge_text=badge_text)

    print(
        colored("\nThat's like", "cyan", attrs=["bold"]),
        colored(f"{len(badges)} badges!", "white", attrs=["bold"]),
        colored("Good job! 🐦", "cyan", attrs=["bold"]),
        file=sys.stderr,
    )

    if args.write:
        with open(args.input, "w") as handle:
            handle.write(output)
    else:
        print(output)



from typing import Type

from .badges._base import Badge
from .models import Project


def get_badge_from_files(badge_class: Type[Badge], project: Project):
    try:
        files = getattr(badge_class, "files")
    except AttributeError:
        files = ()

    for file in files:
        found = list(project.local_path.glob(file))
        if found:
            return badge_class(
                project=project,
            )


def get_badge_from_remotes(badge_class: Type[Badge], project: Project):
    try:
        remotes = getattr(badge_class, "remotes")
    except AttributeError:
        return

    for remote in remotes:
        if project.url.startswith(remote.prefix):
            return badge_class(project=project, remote=remote)
